@@ -1,13 +1,12 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
-from argparse import ArgumentParser, SUPPRESS
+from argparse import ArgumentParser, RawTextHelpFormatter, SUPPRESS
 from os import environ, getcwd
 
 from .. import VERSION_STRING
+from ..utils.cmdline import HELP_get_target_nodes
 from ..utils.text import mark_for_translation as _
 from .apply import bw_apply
 from .debug import bw_debug
+from .diff import bw_diff
 from .groups import bw_groups
 from .hash import bw_hash
 from .items import bw_items
@@ -15,8 +14,7 @@ from .lock import bw_lock_add, bw_lock_remove, bw_lock_show
 from .metadata import bw_metadata
 from .nodes import bw_nodes
 from .plot import bw_plot_group, bw_plot_node, bw_plot_node_groups
-from .repo import bw_repo_bundle_create, bw_repo_create, bw_repo_plugin_install, \
-    bw_repo_plugin_list, bw_repo_plugin_search, bw_repo_plugin_remove, bw_repo_plugin_update
+from .repo import bw_repo_bundle_create, bw_repo_create
 from .run import bw_run
 from .stats import bw_stats
 from .test import bw_test
@@ -37,17 +35,6 @@ def build_parser_bw():
         dest='add_ssh_host_keys',
         help=_("set StrictHostKeyChecking=no instead of yes for SSH"),
     )
-    parser.add_argument(
-        "-A",
-        "--adhoc-nodes",
-        action='store_true',
-        default=False,
-        dest='adhoc_nodes',
-        help=_(
-            "treat unknown node names as adhoc 'virtual' nodes that receive configuration only "
-            "through groups whose member_patterns match the node name given on the command line "
-            "(which also has to be a resolvable hostname)"),
-    )
     parser.add_argument(
         "-d",
         "--debug",
@@ -87,13 +74,19 @@ def build_parser_bw():
 
     # bw apply
     help_apply = _("Applies the configuration defined in your repository to your nodes")
-    parser_apply = subparsers.add_parser("apply", description=help_apply, help=help_apply)
+    parser_apply = subparsers.add_parser(
+        "apply",
+        description=help_apply,
+        help=help_apply,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
+    )
     parser_apply.set_defaults(func=bw_apply)
     parser_apply.add_argument(
-        'target',
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
+        'targets',
+        metavar=_("TARGET"),
+        nargs='+',
         type=str,
-        help=_("target nodes, groups and/or bundle selectors"),
+        help=HELP_get_target_nodes,
     )
     parser_apply.add_argument(
         "-f",
@@ -114,16 +107,18 @@ def build_parser_bw():
     parser_apply.add_argument(
         "-o",
         "--only",
-        default="",
+        default=[],
         dest='autoonly',
-        help=_(
-            "e.g. 'file:/foo,tag:foo,bundle:bar' "
-            "to skip EVERYTHING BUT all instances of file:/foo "
-            "and items with tag 'foo', "
-            "or in bundle 'bar', "
-            "or a dependency of any of these"
-        ),
+        help=_("""skip all items not matching any SELECTOR:
+
+file:/my_path     # this specific item
+tag:my_tag        # items with this tag
+bundle:my_bundle  # items in this bundle
+
+dependencies of selected items will NOT be skipped
+        """),
         metavar=_("SELECTOR"),
+        nargs='+',
         type=str,
     )
     bw_apply_p_default = int(environ.get("BW_NODE_WORKERS", "4"))
@@ -149,15 +144,16 @@ def build_parser_bw():
     parser_apply.add_argument(
         "-s",
         "--skip",
-        default="",
+        default=[],
         dest='autoskip',
-        help=_(
-            "e.g. 'file:/foo,tag:foo,bundle:bar' "
-            "to skip all instances of file:/foo "
-            "and items with tag 'foo', "
-            "or in bundle 'bar'"
-        ),
+        help=_("""skip items matching any SELECTOR:
+
+file:/my_path     # this specific item
+tag:my_tag        # items with this tag
+bundle:my_bundle  # items in this bundle
+        """),
         metavar=_("SELECTOR"),
+        nargs='+',
         type=str,
     )
     parser_apply.add_argument(
@@ -205,10 +201,92 @@ def build_parser_bw():
         help=_("name of node to inspect"),
     )
 
+    # bw diff
+    help_diff = _("Show differences between nodes")
+    parser_diff = subparsers.add_parser(
+        "diff",
+        description=help_diff,
+        help=help_diff,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
+    )
+    parser_diff.set_defaults(func=bw_diff)
+    parser_diff.add_argument(
+        "-b",
+        "--branch",
+        default=None,
+        dest='branch',
+        metavar=_("REV"),
+        required=False,
+        type=str,
+        help=_("compare with this git rev instead (requires clean working dir)"),
+    )
+    parser_diff.add_argument(
+        "-c",
+        "--cmd-change",
+        default=None,
+        dest='cmd_change',
+        metavar=_("CMD_CHANGE"),
+        required=False,
+        type=str,
+        help=_("command to execute between taking metadata snapshots (e.g., change Git branch)"),
+    )
+    parser_diff.add_argument(
+        "-r",
+        "--cmd-reset",
+        default=None,
+        dest='cmd_reset',
+        metavar=_("CMD_RESET"),
+        required=False,
+        type=str,
+        help=_("command to execute when finished (e.g., switch back to original Git branch)"),
+    )
+    parser_diff.add_argument(
+        "-p",
+        "--prompt",
+        action='store_true',
+        default=False,
+        dest='prompt',
+        help=_("interactively ask for user to make changes"),
+    )
+    parser_diff.add_argument(
+        "-i",
+        "--item",
+        default=None,
+        dest='item',
+        metavar=_("ITEM"),
+        required=False,
+        type=str,
+        help=_("compare this specific item between nodes"),
+    )
+    parser_diff.add_argument(
+        "-m",
+        "--metadata",
+        action='store_true',
+        default=False,
+        dest='metadata',
+        help=_("compare metadata instead of configuration"),
+    )
+    parser_diff.add_argument(
+        'targets',
+        metavar=_("TARGET"),
+        nargs='+',
+        type=str,
+        help=HELP_get_target_nodes,
+    )
+
     # bw groups
     help_groups = _("Lists groups in this repository")
     parser_groups = subparsers.add_parser("groups", description=help_groups, help=help_groups)
     parser_groups.set_defaults(func=bw_groups)
+    parser_groups.add_argument(
+        "-a", "--attrs",
+        dest='attrs',
+        metavar=_("ATTR"),
+        nargs='+',
+        type=str,
+        help=_("show table with the given attributes for each group "
+               "(e.g. 'all', 'members', 'os', ...)"),
+    )
     parser_groups.add_argument(
         "-i",
         "--inline",
@@ -219,19 +297,10 @@ def build_parser_bw():
     parser_groups.add_argument(
         'groups',
         default=None,
-        metavar=_("GROUP1,GROUP2..."),
-        nargs='?',
+        metavar=_("GROUP"),
+        nargs='*',
         type=str,
-        help=_("show the given groups and their subgroups"),
-    )
-    parser_groups.add_argument(
-        'attrs',
-        default=None,
-        metavar=_("ATTR1,ATTR2..."),
-        nargs='?',
-        type=str,
-        help=_("show table with the given attributes for each group "
-               "(e.g. 'all', 'members', 'os', ...)"),
+        help=_("show the given groups (and their subgroups, unless --attrs is used)"),
     )
 
     # bw hash
@@ -334,7 +403,11 @@ def build_parser_bw():
 
     # bw lock
     help_lock = _("Manage locks on nodes used to prevent collisions between BundleWrap users")
-    parser_lock = subparsers.add_parser("lock", description=help_lock, help=help_lock)
+    parser_lock = subparsers.add_parser(
+        "lock",
+        description=help_lock,
+        help=help_lock,
+    )
     parser_lock_subparsers = parser_lock.add_subparsers()
 
     # bw lock add
@@ -343,13 +416,15 @@ def build_parser_bw():
         "add",
         description=help_lock_add,
         help=help_lock_add,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
     )
     parser_lock_add.set_defaults(func=bw_lock_add)
     parser_lock_add.add_argument(
-        'target',
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
+        'targets',
+        metavar=_("TARGET"),
+        nargs='+',
         type=str,
-        help=_("target nodes, groups and/or bundle selectors"),
+        help=HELP_get_target_nodes,
     )
     parser_lock_add.add_argument(
         "-c",
@@ -372,10 +447,16 @@ def build_parser_bw():
     parser_lock_add.add_argument(
         "-i",
         "--items",
-        default="*",
+        default=["*"],
         dest='items',
-        help=_("comma-separated list of item selectors the lock applies to "
-               "(defaults to \"*\" meaning all)"),
+        help=_("""lock only items matching any SELECTOR:
+
+file:/my_path     # this specific item
+tag:my_tag        # items with this tag
+bundle:my_bundle  # items in this bundle
+        """),
+        metavar=_("SELECTOR"),
+        nargs='+',
         type=str,
     )
     bw_lock_add_p_default = int(environ.get("BW_NODE_WORKERS", "4"))
@@ -395,13 +476,15 @@ def build_parser_bw():
         "remove",
         description=help_lock_remove,
         help=help_lock_remove,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
     )
     parser_lock_remove.set_defaults(func=bw_lock_remove)
     parser_lock_remove.add_argument(
-        'target',
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
+        'targets',
+        metavar=_("TARGET"),
+        nargs='+',
         type=str,
-        help=_("target nodes, groups and/or bundle selectors"),
+        help=HELP_get_target_nodes,
     )
     parser_lock_remove.add_argument(
         'lock_id',
@@ -426,13 +509,15 @@ def build_parser_bw():
         "show",
         description=help_lock_show,
         help=help_lock_show,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
     )
     parser_lock_show.set_defaults(func=bw_lock_show)
     parser_lock_show.add_argument(
-        'target',
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
+        'targets',
+        metavar=_("TARGETS"),
+        nargs='+',
         type=str,
-        help=_("target node"),
+        help=HELP_get_target_nodes,
     )
     bw_lock_show_p_default = int(environ.get("BW_NODE_WORKERS", "4"))
     parser_lock_show.add_argument(
@@ -446,48 +531,69 @@ def build_parser_bw():
     )
 
     # bw metadata
-    help_metadata = ("View a JSON representation of a node's metadata")
+    help_metadata = ("View a JSON representation of a node's metadata (defaults blue, reactors green, groups yellow, node red, uncolored if mixed-source) or a table of selected metadata keys from multiple nodes")
     parser_metadata = subparsers.add_parser(
         "metadata",
         description=help_metadata,
         help=help_metadata,
+        formatter_class=RawTextHelpFormatter,
     )
     parser_metadata.set_defaults(func=bw_metadata)
     parser_metadata.add_argument(
-        'target',
-        metavar=_("NODE"),
+        'targets',
+        metavar=_("TARGET"),
+        nargs='+',
         type=str,
-        help=_("node to print JSON-formatted metadata for"),
+        help=HELP_get_target_nodes,
     )
     parser_metadata.add_argument(
-        'keys',
+        "-k", "--keys",
         default=[],
+        dest='keys',
         metavar=_("KEY"),
         nargs='*',
         type=str,
-        help=_("print only partial metadata from the given space-separated key path (e.g. `bw metadata mynode users jdoe` to show `mynode.metadata['users']['jdoe']`)"),
+        help=_("show only partial metadata from the given key paths (e.g. `bw metadata mynode -k users/jdoe` to show `mynode.metadata['users']['jdoe']`)"),
     )
     parser_metadata.add_argument(
-        "--blame",
+        "-b", "--blame",
         action='store_true',
         dest='blame',
         help=_("show where each piece of metadata comes from"),
     )
     parser_metadata.add_argument(
-        "-t",
-        "--table",
+        "-D", "--hide-defaults",
         action='store_true',
-        dest='table',
-        help=_(
-            "show a table of selected metadata values from multiple nodes instead; "
-            "allows for multiple comma-separated paths in KEY; "
-            "allows for node selectors in NODE (e.g. 'NODE1,NODE2,GROUP1,bundle:BUNDLE1...')"
-        ),
+        dest='hide_defaults',
+        help=_("hide values set by defaults in metadata.py"),
+    )
+    parser_metadata.add_argument(
+        "-G", "--hide-groups",
+        action='store_true',
+        dest='hide_groups',
+        help=_("hide values set in groups.py"),
+    )
+    parser_metadata.add_argument(
+        "-N", "--hide-node",
+        action='store_true',
+        dest='hide_node',
+        help=_("hide values set in nodes.py"),
+    )
+    parser_metadata.add_argument(
+        "-R", "--hide-reactors",
+        action='store_true',
+        dest='hide_reactors',
+        help=_("hide values set by reactors in metadata.py"),
     )
 
     # bw nodes
     help_nodes = _("List nodes in this repository")
-    parser_nodes = subparsers.add_parser("nodes", description=help_nodes, help=help_nodes)
+    parser_nodes = subparsers.add_parser(
+        "nodes",
+        description=help_nodes,
+        help=help_nodes,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
+    )
     parser_nodes.set_defaults(func=bw_nodes)
     parser_nodes.add_argument(
         "-i",
@@ -497,18 +603,20 @@ def build_parser_bw():
         help=_("keep lists on a single line (for grep)"),
     )
     parser_nodes.add_argument(
-        'target',
+        'targets',
         default=None,
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
-        nargs='?',
+        metavar=_("TARGET"),
+        nargs='*',
         type=str,
-        help=_("filter according to nodes, groups and/or bundle selectors"),
+        help=HELP_get_target_nodes,
     )
     parser_nodes.add_argument(
-        'attrs',
+        "-a",
+        "--attrs",
         default=None,
-        metavar=_("ATTR1,ATTR2..."),
-        nargs='?',
+        dest='attrs',
+        metavar=_("ATTR"),
+        nargs='+',
         type=str,
         help=_("show table with the given attributes for each node "
                "(e.g. 'all', 'groups', 'bundles', 'hostname', 'os', ...)"),
@@ -588,12 +696,6 @@ def build_parser_bw():
         dest='depends_reverse',
         help=_("do not show reverse dependencies ('needed_by')"),
     )
-    parser_plot_subparsers_node.add_argument(
-        "--no-depends-static",
-        action='store_false',
-        dest='depends_static',
-        help=_("do not show static dependencies"),
-    )
 
     # bw plot groups-for-node
     help_plot_node_groups = _("Show where a specific node gets its groups from")
@@ -634,94 +736,21 @@ def build_parser_bw():
     parser_repo_subparsers_create = parser_repo_subparsers.add_parser("create")
     parser_repo_subparsers_create.set_defaults(func=bw_repo_create)
 
-    # bw repo plugin
-    parser_repo_subparsers_plugin = parser_repo_subparsers.add_parser("plugin")
-    parser_repo_subparsers_plugin_subparsers = parser_repo_subparsers_plugin.add_subparsers()
-
-    # bw repo plugin install
-    parser_repo_subparsers_plugin_install = parser_repo_subparsers_plugin_subparsers.add_parser("install")
-    parser_repo_subparsers_plugin_install.set_defaults(func=bw_repo_plugin_install)
-    parser_repo_subparsers_plugin_install.add_argument(
-        'plugin',
-        metavar=_("PLUGIN_NAME"),
-        type=str,
-        help=_("name of plugin to install"),
-    )
-    parser_repo_subparsers_plugin_install.add_argument(
-        "-f",
-        "--force",
-        action='store_true',
-        dest='force',
-        help=_("overwrite existing files when installing"),
-    )
-
-    # bw repo plugin list
-    parser_repo_subparsers_plugin_list = parser_repo_subparsers_plugin_subparsers.add_parser("list")
-    parser_repo_subparsers_plugin_list.set_defaults(func=bw_repo_plugin_list)
-
-    # bw repo plugin remove
-    parser_repo_subparsers_plugin_remove = parser_repo_subparsers_plugin_subparsers.add_parser("remove")
-    parser_repo_subparsers_plugin_remove.set_defaults(func=bw_repo_plugin_remove)
-    parser_repo_subparsers_plugin_remove.add_argument(
-        'plugin',
-        metavar=_("PLUGIN_NAME"),
-        type=str,
-        help=_("name of plugin to remove"),
-    )
-    parser_repo_subparsers_plugin_remove.add_argument(
-        "-f",
-        "--force",
-        action='store_true',
-        dest='force',
-        help=_("remove files even if locally modified"),
-    )
-
-    # bw repo plugin search
-    parser_repo_subparsers_plugin_search = parser_repo_subparsers_plugin_subparsers.add_parser("search")
-    parser_repo_subparsers_plugin_search.set_defaults(func=bw_repo_plugin_search)
-    parser_repo_subparsers_plugin_search.add_argument(
-        'term',
-        metavar=_("SEARCH_STRING"),
-        nargs='?',
-        type=str,
-        help=_("look for this string in plugin names and descriptions"),
-    )
-
-    # bw repo plugin update
-    parser_repo_subparsers_plugin_update = parser_repo_subparsers_plugin_subparsers.add_parser("update")
-    parser_repo_subparsers_plugin_update.set_defaults(func=bw_repo_plugin_update)
-    parser_repo_subparsers_plugin_update.add_argument(
-        'plugin',
-        default=None,
-        metavar=_("PLUGIN_NAME"),
-        nargs='?',
-        type=str,
-        help=_("name of plugin to update"),
-    )
-    parser_repo_subparsers_plugin_update.add_argument(
-        "-c",
-        "--check-only",
-        action='store_true',
-        dest='check_only',
-        help=_("only show what would be updated"),
-    )
-    parser_repo_subparsers_plugin_update.add_argument(
-        "-f",
-        "--force",
-        action='store_true',
-        dest='force',
-        help=_("overwrite local modifications when updating"),
-    )
-
     # bw run
     help_run = _("Run a one-off command on a number of nodes")
-    parser_run = subparsers.add_parser("run", description=help_run, help=help_run)
+    parser_run = subparsers.add_parser(
+        "run",
+        description=help_run,
+        help=help_run,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
+    )
     parser_run.set_defaults(func=bw_run)
     parser_run.add_argument(
-        'target',
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
+        'targets',
+        metavar=_("TARGET"),
+        nargs='+',
         type=str,
-        help=_("target nodes, groups and/or bundle selectors"),
+        help=HELP_get_target_nodes,
     )
     parser_run.add_argument(
         'command',
@@ -785,22 +814,20 @@ def build_parser_bw():
                   "change in future releases). Currently, the default is -IJKM "
                   "if specific nodes are given and -HIJKMS if testing the "
                   "entire repo.")
-    parser_test = subparsers.add_parser("test", description=help_test, help=help_test)
+    parser_test = subparsers.add_parser(
+        "test",
+        description=help_test,
+        help=help_test,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
+    )
     parser_test.set_defaults(func=bw_test)
     parser_test.add_argument(
-        'target',
+        'targets',
         default=None,
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
-        nargs='?',
+        metavar=_("TARGET"),
+        nargs='*',
         type=str,
-        help=_("target nodes, groups and/or bundle selectors (defaults to all)"),
-    )
-    parser_test.add_argument(
-        "-c",
-        "--plugin-conflicts",
-        action='store_true',
-        dest='plugin_conflicts',
-        help=_("check for local modifications to files installed by plugins"),
+        help=HELP_get_target_nodes + _("\n(defaults to all)"),
     )
     parser_test.add_argument(
         "-d",
@@ -866,10 +893,10 @@ def build_parser_bw():
     )
     parser_test.add_argument(
         "-M",
-        "--metadata-collisions",
+        "--metadata-conflicts",
         action='store_true',
-        dest='metadata_collisions',
-        help=_("check for conflicting metadata keys in group metadata"),
+        dest='metadata_conflicts',
+        help=_("check for conflicting metadata keys in group metadata, reactors, and defaults"),
     )
     parser_test.add_argument(
         "-o",
@@ -879,17 +906,11 @@ def build_parser_bw():
         help=_("check for bundles not assigned to any node"),
     )
     parser_test.add_argument(
-        "-s",
-        "--secret-rotation",
-        default=None,
-        dest='ignore_secret_identifiers',
-        help=_("ensure every string passed to repo.vault.[human_]password_for() is used at least "
-               "twice (using it only once means you're probably managing only one end of an "
-               "authentication, making it dangerous to rotate your .secrets.cfg); PATTERNS is a "
-               "comma-separated list of regex patterns for strings to ignore in this check "
-               "(just pass an empty string if you don't need to ignore anything)"),
-        metavar="PATTERNS",
-        type=str,
+        "-q",
+        "--quiet",
+        action='store_true',
+        dest='quiet',
+        help=_("don't show successful tests"),
     )
     parser_test.add_argument(
         "-S",
@@ -901,13 +922,19 @@ def build_parser_bw():
 
     # bw verify
     help_verify = _("Inspect the health or 'correctness' of a node without changing it")
-    parser_verify = subparsers.add_parser("verify", description=help_verify, help=help_verify)
+    parser_verify = subparsers.add_parser(
+        "verify",
+        description=help_verify,
+        help=help_verify,
+        formatter_class=RawTextHelpFormatter,  # for HELP_get_target_nodes
+    )
     parser_verify.set_defaults(func=bw_verify)
     parser_verify.add_argument(
-        'target',
-        metavar=_("NODE1,NODE2,GROUP1,bundle:BUNDLE1..."),
+        'targets',
+        metavar=_("TARGET"),
+        nargs='+',
         type=str,
-        help=_("target nodes, groups and/or bundle selectors"),
+        help=HELP_get_target_nodes,
     )
     parser_verify.add_argument(
         "-a",
