@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from ..deps import prepare_dependencies
 from ..utils.plot import graph_for_items, plot_group, plot_node_groups
 from ..utils.cmdline import get_group, get_node
@@ -26,13 +23,12 @@ def bw_plot_group(repo, args):
 
 
 def bw_plot_node(repo, args):
-    node = get_node(repo, args['node'], adhoc_nodes=args['adhoc_nodes'])
+    node = get_node(repo, args['node'])
     for line in graph_for_items(
         node.name,
         prepare_dependencies(node.items, node.os, node.os_version),
         cluster=args['cluster'],
         concurrency=args['depends_concurrency'],
-        static=args['depends_static'],
         regular=args['depends_regular'],
         reverse=args['depends_reverse'],
         auto=args['depends_auto'],
@@ -41,6 +37,6 @@ def bw_plot_node(repo, args):
 
 
 def bw_plot_node_groups(repo, args):
-    node = get_node(repo, args['node'], adhoc_nodes=args['adhoc_nodes'])
+    node = get_node(repo, args['node'])
     for line in plot_node_groups(node):
         io.stdout(line)
