@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from datetime import datetime
 from sys import exit
 
@@ -116,7 +113,7 @@ def stats_summary(node_stats, total_duration):
 def bw_verify(repo, args):
     errors = []
     node_stats = {}
-    pending_nodes = get_target_nodes(repo, args['target'], adhoc_nodes=args['adhoc_nodes'])
+    pending_nodes = get_target_nodes(repo, args['targets'])
     start_time = datetime.now()
     io.progress_set_total(count_items(pending_nodes))
 
