@@ -1,12 +1,20 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
+from os import mkdir
+from os.path import exists, join
 import re
 
+from tomlkit import dumps as toml_dump, parse as toml_parse
+
 from .exceptions import NoSuchGroup, NoSuchNode, RepositoryError
-from .utils import cached_property, names
-from .utils.dicts import hash_statedict
-from .utils.text import mark_for_translation as _, validate_name
+from .utils import cached_property, error_context, get_file_contents, names
+from .utils.dicts import (
+    dict_to_toml,
+    hash_statedict,
+    set_key_at_path,
+    validate_dict,
+    COLLECTION_OF_STRINGS,
+    TUPLE_OF_INTS,
+)
+from .utils.text import mark_for_translation as _, toml_clean, validate_name
 
 
 GROUP_ATTR_DEFAULTS = {
@@ -14,6 +22,7 @@ GROUP_ATTR_DEFAULTS = {
     'cmd_wrapper_outer': "sudo sh -c {}",
     'dummy': False,
     'kubectl_context': None,
+    'locking_node': None,
     'os': 'linux',
     # Setting os_version to 0 by default will probably yield less
     # surprises than setting it to max_int. Users will probably
@@ -33,6 +42,24 @@ GROUP_ATTR_DEFAULTS = {
     'use_shadow_passwords': True,
 }
 
+GROUP_ATTR_TYPES = {
+    'bundles': COLLECTION_OF_STRINGS,
+    'cmd_wrapper_inner': str,
+    'cmd_wrapper_outer': str,
+    'dummy': bool,
+    'file_path': str,
+    'kubectl_context': (str, type(None)),
+    'locking_node': (str, type(None)),
+    'member_patterns': COLLECTION_OF_STRINGS,
+    'members': COLLECTION_OF_STRINGS,
+    'metadata': dict,
+    'os': str,
+    'os_version': TUPLE_OF_INTS,
+    'subgroups': COLLECTION_OF_STRINGS,
+    'subgroup_patterns': COLLECTION_OF_STRINGS,
+    'use_shadow_passwords': bool,
+}
+
 
 def _build_error_chain(loop_node, last_node, nodes_in_between):
     """
@@ -54,30 +81,35 @@ def _build_error_chain(loop_node, last_node, nodes_in_between):
     return error_chain
 
 
-class Group(object):
+class Group:
     """
     A group of nodes.
     """
-    def __init__(self, group_name, infodict=None):
-        if infodict is None:
-            infodict = {}
+    def __init__(self, group_name, attributes=None):
+        if attributes is None:
+            attributes = {}
 
         if not validate_name(group_name):
             raise RepositoryError(_("'{}' is not a valid group name.").format(group_name))
 
+        with error_context(group_name=group_name):
+            validate_dict(attributes, GROUP_ATTR_TYPES)
+
+        self._attributes = attributes
+        self._immediate_subgroup_patterns = {
+            re.compile(pattern) for pattern in
+            set(attributes.get('subgroup_patterns', set()))
+        }
+        self._member_patterns = {
+            re.compile(pattern) for pattern in
+            set(attributes.get('member_patterns', set()))
+        }
         self.name = group_name
-        self.bundle_names = infodict.get('bundles', [])
-        self.immediate_subgroup_names = infodict.get('subgroups', [])
-        self.immediate_subgroup_patterns = infodict.get('subgroup_patterns', [])
-        self.members_add = infodict.get('members_add', None)
-        self.members_remove = infodict.get('members_remove', None)
-        self.metadata = infodict.get('metadata', {})
-        self.node_patterns = infodict.get('member_patterns', [])
-        self.static_member_names = infodict.get('members', [])
+        self.file_path = attributes.get('file_path')
 
         for attr in GROUP_ATTR_DEFAULTS:
             # defaults are applied in node.py
-            setattr(self, attr, infodict.get(attr))
+            setattr(self, attr, attributes.get(attr))
 
     def __lt__(self, other):
         return self.name < other.name
@@ -114,23 +146,8 @@ class Group(object):
                 yield node
 
     @cached_property
-    def _static_nodes(self):
-        result = set()
-        result.update(self._nodes_from_members)
-        result.update(self._nodes_from_patterns)
-        return result
-
-    @property
-    def _subgroup_names_from_patterns(self):
-        for pattern in self.immediate_subgroup_patterns:
-            compiled_pattern = re.compile(pattern)
-            for group in self.repo.groups:
-                if compiled_pattern.search(group.name) is not None and group != self:
-                    yield group.name
-
-    @property
     def _nodes_from_members(self):
-        for node_name in self.static_member_names:
+        for node_name in self._attributes.get('members', set()):
             try:
                 yield self.repo.get_node(node_name)
             except NoSuchNode:
@@ -143,19 +160,18 @@ class Group(object):
                 ))
 
     @property
-    def _nodes_from_patterns(self):
-        for pattern in self.node_patterns:
-            compiled_pattern = re.compile(pattern)
-            for node in self.repo.nodes:
-                if not compiled_pattern.search(node.name) is None:
-                    yield node
+    def _subgroup_names_from_patterns(self):
+        for pattern in self._immediate_subgroup_patterns:
+            for group in self.repo.groups:
+                if pattern.search(group.name) is not None and group != self:
+                    yield group.name
 
     def _check_subgroup_names(self, visited_names):
         """
         Recursively finds subgroups and checks for loops.
         """
         for name in set(
-            list(self.immediate_subgroup_names) +
+            list(self._attributes.get('subgroups', set())) +
             list(self._subgroup_names_from_patterns)
         ):
             if name not in visited_names:
@@ -209,13 +225,37 @@ class Group(object):
         for group_name in set(self._check_subgroup_names([self.name])):
             yield self.repo.get_group(group_name)
 
+    @cached_property
+    def toml(self):
+        if not self.file_path or not self.file_path.endswith(".toml"):
+            raise ValueError(_("group {} not in TOML format").format(self.name))
+        return toml_parse(get_file_contents(self.file_path))
+
+    def toml_save(self):
+        try:
+            toml_doc = self.toml
+        except ValueError:
+            attributes = self._attributes.copy()
+            del attributes['file_path']
+            toml_doc = dict_to_toml(attributes)
+            self.file_path = join(self.repo.path, "groups", self.name + ".toml")
+        if not exists(join(self.repo.path, "groups")):
+            mkdir(join(self.repo.path, "groups"))
+        with open(self.file_path, 'w') as f:
+            f.write(toml_clean(toml_dump(toml_doc)))
+
+    def toml_set(self, path, value):
+        if not isinstance(path, tuple):
+            path = path.split("/")
+        set_key_at_path(self.toml, path, value)
+
     @cached_property
     def immediate_subgroups(self):
         """
         Iterator over all immediate subgroups as group objects.
         """
         for group_name in set(
-            list(self.immediate_subgroup_names) +
+            list(self._attributes.get('subgroups', set())) +
             list(self._subgroup_names_from_patterns)
         ):
             try:
