@@ -1,9 +1,7 @@
-# -*- coding: utf-8 -*-
 """
 Note that modules in this package have to use absolute imports because
 Repository.item_classes loads them as files.
 """
-from __future__ import unicode_literals
 from copy import copy
 from datetime import datetime
 from inspect import cleandoc
@@ -16,6 +14,7 @@ from bundlewrap.utils.dicts import diff_keys, diff_value, hash_statedict, valida
 from bundlewrap.utils.text import force_text, mark_for_translation as _
 from bundlewrap.utils.text import blue, bold, italic, wrap_question
 from bundlewrap.utils.ui import io
+from bundlewrap.operations import run_local
 
 
 BUILTIN_ITEM_ATTRIBUTES = {
@@ -50,7 +49,7 @@ def format_comment(comment):
     return result
 
 
-class ItemStatus(object):
+class ItemStatus:
     """
     Holds information on a particular Item such as whether it needs
     fixing and what's broken.
@@ -90,11 +89,10 @@ def make_normalize(attribute_default):
         return copy
 
 
-class Item(object):
+class Item:
     """
     A single piece of configuration (e.g. a file, a package, a service).
     """
-    BLOCK_CONCURRENT = []
     BUNDLE_ATTRIBUTE_NAME = None
     ITEM_ATTRIBUTES = {}
     ITEM_TYPE_NAME = None
@@ -151,6 +149,7 @@ class Item(object):
         self.name = name
         self.node = bundle.node
         self.when_creating = {}
+        self._command_results = []
         self._faults_missing_for_attributes = set()
         self._precedes_items = []
 
@@ -350,23 +349,21 @@ class Item(object):
         skipped based on the given set of locks.
         """
         for lock in mine:
-            for selector in lock['items']:
-                if self.covered_by_autoskip_selector(selector):
-                    io.debug(_("{item} on {node} whitelisted by lock {lock}").format(
-                        item=self.id,
-                        lock=lock['id'],
-                        node=self.node.name,
-                    ))
-                    return False
+            if self.covered_by_autoskip_selector(lock['items']):
+                io.debug(_("{item} on {node} whitelisted by lock {lock}").format(
+                    item=self.id,
+                    lock=lock['id'],
+                    node=self.node.name,
+                ))
+                return False
         for lock in others:
-            for selector in lock['items']:
-                if self.covered_by_autoskip_selector(selector):
-                    io.debug(_("{item} on {node} blacklisted by lock {lock}").format(
-                        item=self.id,
-                        lock=lock['id'],
-                        node=self.node.name,
-                    ))
-                    return True
+            if self.covered_by_autoskip_selector(lock['items']):
+                io.debug(_("{item} on {node} blacklisted by lock {lock}").format(
+                    item=self.id,
+                    lock=lock['id'],
+                    node=self.node.name,
+                ))
+                return True
         return False
 
     def _test(self):
@@ -425,6 +422,7 @@ class Item(object):
                 type=cls.ITEM_TYPE_NAME,
             ))
 
+    @classmethod
     def _validate_required_attributes(cls, bundle, item_id, attributes):
         missing = []
         for attrname in cls.REQUIRED_ATTRIBUTES:
@@ -441,8 +439,8 @@ class Item(object):
 
     def apply(
         self,
-        autoskip_selector="",
-        autoonly_selector="",
+        autoskip_selector=[],
+        autoonly_selector=[],
         my_soft_locks=(),
         other_peoples_soft_locks=(),
         interactive=False,
@@ -617,6 +615,22 @@ class Item(object):
         )
         return (status_code, details)
 
+    def run_local(self, command, **kwargs):
+        result = run_local(command, **kwargs)
+        self._command_results.append({
+            'command': command,
+            'result': result,
+        })
+        return result
+
+    def run(self, command, **kwargs):
+        result = self.node.run(command, **kwargs)
+        self._command_results.append({
+            'command': command,
+            'result': result,
+        })
+        return result
+
     def ask(self, status_should, status_actual, relevant_keys):
         """
         Returns a string asking the user if this item should be
@@ -642,7 +656,7 @@ class Item(object):
         True if this item should be skipped based on the given selector
         string (e.g. "tag:foo,bundle:bar").
         """
-        components = [c.strip() for c in autoskip_selector.split(",")]
+        components = [c.strip() for c in autoskip_selector]
         if (
             "*" in components or
             self.id in components or
@@ -662,7 +676,7 @@ class Item(object):
         """
         if not autoonly_selector:
             return True
-        components = [c.strip() for c in autoonly_selector.split(",")]
+        components = [c.strip() for c in autoonly_selector]
         if (
             self.id in components or
             "bundle:{}".format(self.bundle.name) in components or
