@@ -1,20 +1,16 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from abc import ABCMeta
 import json
 from os.path import exists, join
 import re
 
 from bundlewrap.exceptions import BundleError
-from bundlewrap.metadata import MetadataJSONEncoder
+from bundlewrap.metadata import metadata_to_json
 from bundlewrap.operations import run_local
 from bundlewrap.items import BUILTIN_ITEM_ATTRIBUTES, Item
 from bundlewrap.items.files import content_processor_jinja2, content_processor_mako
 from bundlewrap.utils.dicts import merge_dict, reduce_dict
 from bundlewrap.utils.ui import io
 from bundlewrap.utils.text import force_text, mark_for_translation as _
-from six import add_metaclass
 import yaml
 
 
@@ -24,8 +20,7 @@ def log_error(run_result):
         io.debug(run_result.stderr.decode('utf-8'))
 
 
-@add_metaclass(ABCMeta)
-class KubernetesItem(Item):
+class KubernetesItem(Item, metaclass=ABCMeta):
     """
     A generic Kubernetes item.
     """
@@ -38,7 +33,6 @@ class KubernetesItem(Item):
         'context': None,
     }
     KIND = None
-    KUBERNETES_APIVERSION = "v1"
     NAME_REGEX = r"^[a-z0-9-\.]{1,253}/[a-z0-9-\.]{1,253}$"
     NAME_REGEX_COMPILED = re.compile(NAME_REGEX)
 
@@ -59,14 +53,17 @@ class KubernetesItem(Item):
         if self.attributes['delete']:
             return None
         else:
-            return {'manifest': self.manifest}
+            return {'manifest': json.dumps(
+                self.nuke_changing_attributes(json.loads(self.manifest)),
+                indent=4, sort_keys=True,
+            )}
 
     def fix(self, status):
         if status.must_be_deleted:
-            result = run_local(self._kubectl + ["delete", self.KIND, self.resource_name])
+            result = self.run_local(self._kubectl + ["delete", self.KIND, self.resource_name])
             log_error(result)
         else:
-            result = run_local(
+            result = self.run_local(
                 self._kubectl + ["apply", "-f", "-"],
                 data_stdin=self.manifest.encode('utf-8'),
             )
@@ -128,7 +125,6 @@ class KubernetesItem(Item):
 
         merged_manifest = merge_dict(
             {
-                'apiVersion': self.KUBERNETES_APIVERSION,
                 'kind': self.KIND,
                 'metadata': {
                     'name': self.name.split("/")[-1],
@@ -137,7 +133,7 @@ class KubernetesItem(Item):
             user_manifest,
         )
 
-        if merged_manifest['apiVersion'] is None:
+        if merged_manifest.get('apiVersion') is None:
             raise BundleError(_(
                 "{item} from bundle '{bundle}' needs an apiVersion in its manifest"
             ).format(item=self.id, bundle=self.bundle.name))
@@ -146,17 +142,20 @@ class KubernetesItem(Item):
 
     @property
     def manifest(self):
-        return json.dumps(
-            self._manifest_dict,
-            cls=MetadataJSONEncoder,
-            indent=4,
-            sort_keys=True,
-        )
+        return metadata_to_json(self._manifest_dict)
 
     @property
     def namespace(self):
         return self.name.split("/", 1)[0] or None
 
+    def nuke_changing_attributes(self, manifest):
+        # These attributes cannot be managed by us. The cluster decides
+        # what's going on. We can't fix it.
+        for i in ('apiVersion', 'status'):
+            if i in manifest:
+                del manifest[i]
+        return manifest
+
     def patch_attributes(self, attributes):
         if 'context' not in attributes:
             attributes['context'] = {}
@@ -172,7 +171,7 @@ class KubernetesItem(Item):
         return self._manifest_dict['metadata']['name']
 
     def sdict(self):
-        result = run_local(self._kubectl + ["get", "-o", "json", self.KIND, self.resource_name])
+        result = self.run_local(self._kubectl + ["get", "-o", "json", self.KIND, self.resource_name])
         if result.return_code == 0:
             full_json_response = json.loads(result.stdout.decode('utf-8'))
             if full_json_response.get("status", {}).get("phase") == "Terminating":
@@ -180,7 +179,7 @@ class KubernetesItem(Item):
                 return None
             return {'manifest': json.dumps(reduce_dict(
                 full_json_response,
-                json.loads(self.manifest),
+                self.nuke_changing_attributes(json.loads(self.manifest)),
             ), indent=4, sort_keys=True)}
         elif result.return_code == 1 and "NotFound" in result.stderr.decode('utf-8'):
             return None
@@ -226,7 +225,6 @@ class KubernetesItem(Item):
 class KubernetesRawItem(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_raw"
     ITEM_TYPE_NAME = "k8s_raw"
-    KUBERNETES_APIVERSION = None
     NAME_REGEX = r"^([a-z0-9-\.]{1,253})?/[a-zA-Z0-9-\.]{1,253}/[a-z0-9-\.]{1,253}$"
     NAME_REGEX_COMPILED = re.compile(NAME_REGEX)
 
@@ -244,6 +242,7 @@ class KubernetesRawItem(KubernetesItem):
                     bundle=self.bundle.name,
                     item2=item.id,
                     bundle2=item.bundle.name,
+                    node=self.node.name,
                 ))
 
     def get_auto_deps(self, items):
@@ -264,7 +263,6 @@ class KubernetesRawItem(KubernetesItem):
 class KubernetesClusterRole(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_clusterroles"
     KIND = "ClusterRole"
-    KUBERNETES_APIVERSION = "rbac.authorization.k8s.io/v1"
     ITEM_TYPE_NAME = "k8s_clusterrole"
     NAME_REGEX = r"^[a-z0-9-\.]{1,253}$"
     NAME_REGEX_COMPILED = re.compile(NAME_REGEX)
@@ -277,7 +275,6 @@ class KubernetesClusterRole(KubernetesItem):
 class KubernetesClusterRoleBinding(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_clusterrolebindings"
     KIND = "ClusterRoleBinding"
-    KUBERNETES_APIVERSION = "rbac.authorization.k8s.io/v1"
     ITEM_TYPE_NAME = "k8s_clusterrolebinding"
     NAME_REGEX = r"^[a-z0-9-\.]{1,253}$"
     NAME_REGEX_COMPILED = re.compile(NAME_REGEX)
@@ -295,21 +292,18 @@ class KubernetesClusterRoleBinding(KubernetesItem):
 class KubernetesConfigMap(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_configmaps"
     KIND = "ConfigMap"
-    KUBERNETES_APIVERSION = "v1"
     ITEM_TYPE_NAME = "k8s_configmap"
 
 
 class KubernetesCronJob(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_cronjobs"
     KIND = "CronJob"
-    KUBERNETES_APIVERSION = "batch/v1beta1"
     ITEM_TYPE_NAME = "k8s_cronjob"
 
 
 class KubernetesCustomResourceDefinition(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_crd"
     KIND = "CustomResourceDefinition"
-    KUBERNETES_APIVERSION = "apiextensions.k8s.io/v1beta1"
     ITEM_TYPE_NAME = "k8s_crd"
     NAME_REGEX = r"^[a-z0-9-\.]{1,253}$"
     NAME_REGEX_COMPILED = re.compile(NAME_REGEX)
@@ -325,7 +319,6 @@ class KubernetesCustomResourceDefinition(KubernetesItem):
 class KubernetesDaemonSet(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_daemonsets"
     KIND = "DaemonSet"
-    KUBERNETES_APIVERSION = "v1"
     ITEM_TYPE_NAME = "k8s_daemonset"
 
     def get_auto_deps(self, items):
@@ -342,7 +335,6 @@ class KubernetesDaemonSet(KubernetesItem):
 class KubernetesDeployment(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_deployments"
     KIND = "Deployment"
-    KUBERNETES_APIVERSION = "extensions/v1beta1"
     ITEM_TYPE_NAME = "k8s_deployment"
 
     def get_auto_deps(self, items):
@@ -359,7 +351,6 @@ class KubernetesDeployment(KubernetesItem):
 class KubernetesIngress(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_ingresses"
     KIND = "Ingress"
-    KUBERNETES_APIVERSION = "extensions/v1beta1"
     ITEM_TYPE_NAME = "k8s_ingress"
 
     def get_auto_deps(self, items):
@@ -376,7 +367,6 @@ class KubernetesIngress(KubernetesItem):
 class KubernetesNamespace(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_namespaces"
     KIND = "Namespace"
-    KUBERNETES_APIVERSION = "v1"
     ITEM_TYPE_NAME = "k8s_namespace"
     NAME_REGEX = r"^[a-z0-9-\.]{1,253}$"
     NAME_REGEX_COMPILED = re.compile(NAME_REGEX)
@@ -388,7 +378,6 @@ class KubernetesNamespace(KubernetesItem):
 class KubernetesNetworkPolicy(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_networkpolicies"
     KIND = "NetworkPolicy"
-    KUBERNETES_APIVERSION = "networking.k8s.io/v1"
     ITEM_TYPE_NAME = "k8s_networkpolicy"
     NAME_REGEX = r"^([a-z0-9-\.]{1,253})?/[a-z0-9-\.]{1,253}$"
     NAME_REGEX_COMPILED = re.compile(NAME_REGEX)
@@ -397,21 +386,18 @@ class KubernetesNetworkPolicy(KubernetesItem):
 class KubernetesPersistentVolumeClain(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_pvc"
     KIND = "PersistentVolumeClaim"
-    KUBERNETES_APIVERSION = "v1"
     ITEM_TYPE_NAME = "k8s_pvc"
 
 
 class KubernetesRole(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_roles"
     KIND = "Role"
-    KUBERNETES_APIVERSION = "rbac.authorization.k8s.io/v1"
     ITEM_TYPE_NAME = "k8s_role"
 
 
 class KubernetesRoleBinding(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_rolebindings"
     KIND = "RoleBinding"
-    KUBERNETES_APIVERSION = "rbac.authorization.k8s.io/v1"
     ITEM_TYPE_NAME = "k8s_rolebinding"
 
     def get_auto_deps(self, items):
@@ -423,7 +409,6 @@ class KubernetesRoleBinding(KubernetesItem):
 class KubernetesSecret(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_secrets"
     KIND = "Secret"
-    KUBERNETES_APIVERSION = "v1"
     ITEM_TYPE_NAME = "k8s_secret"
 
     def get_auto_deps(self, items):
@@ -433,21 +418,18 @@ class KubernetesSecret(KubernetesItem):
 class KubernetesService(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_services"
     KIND = "Service"
-    KUBERNETES_APIVERSION = "v1"
     ITEM_TYPE_NAME = "k8s_service"
 
 
 class KubernetesServiceAccount(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_serviceaccounts"
     KIND = "ServiceAccount"
-    KUBERNETES_APIVERSION = "v1"
     ITEM_TYPE_NAME = "k8s_serviceaccount"
 
 
 class KubernetesStatefulSet(KubernetesItem):
     BUNDLE_ATTRIBUTE_NAME = "k8s_statefulsets"
     KIND = "StatefulSet"
-    KUBERNETES_APIVERSION = "apps/v1"
     ITEM_TYPE_NAME = "k8s_statefulset"
 
     def get_auto_deps(self, items):
