@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from os.path import exists, join
 
 from bundlewrap.utils.testing import host_os, make_repo, run
@@ -37,7 +34,7 @@ def test_only_bundle_with_dep(tmpdir):
         },
     )
 
-    run("bw apply -o bundle:test localhost", path=str(tmpdir))
+    run("bw apply -o bundle:test -- localhost", path=str(tmpdir))
     assert exists(join(str(tmpdir), "foo"))
     assert exists(join(str(tmpdir), "bar"))
     assert not exists(join(str(tmpdir), "baz"))
