@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from os.path import exists, join
 
 from bundlewrap.utils.testing import host_os, make_repo, run
@@ -25,7 +22,7 @@ def test_skip_bundle(tmpdir):
             },
         },
     )
-    result = run("bw apply --skip bundle:test localhost", path=str(tmpdir))
+    result = run("bw apply --skip bundle:test -- localhost", path=str(tmpdir))
     assert result[2] == 0
     assert not exists(join(str(tmpdir), "foo"))
 
@@ -45,14 +42,15 @@ def test_skip_group(tmpdir):
         nodes={
             "localhost": {
                 'bundles': ["test"],
+                'groups': {"foo"},
                 'os': host_os(),
             },
         },
         groups={
-            "foo": {'members': ["localhost"]},
+            "foo": {},
         },
     )
-    result = run("bw apply --skip group:foo localhost", path=str(tmpdir))
+    result = run("bw apply --skip group:foo -- localhost", path=str(tmpdir))
     assert result[2] == 0
     assert not exists(join(str(tmpdir), "foo"))
 
@@ -76,7 +74,7 @@ def test_skip_id(tmpdir):
             },
         },
     )
-    result = run("bw apply --skip file:{} localhost".format(join(str(tmpdir), "foo")), path=str(tmpdir))
+    result = run("bw apply --skip file:{} -- localhost".format(join(str(tmpdir), "foo")), path=str(tmpdir))
     assert result[2] == 0
     assert not exists(join(str(tmpdir), "foo"))
 
@@ -100,7 +98,7 @@ def test_skip_node(tmpdir):
             },
         },
     )
-    result = run("bw apply --skip node:localhost localhost", path=str(tmpdir))
+    result = run("bw apply --skip node:localhost -- localhost", path=str(tmpdir))
     assert result[2] == 0
     assert not exists(join(str(tmpdir), "foo"))
 
@@ -125,7 +123,7 @@ def test_skip_tag(tmpdir):
             },
         },
     )
-    result = run("bw apply --skip tag:nope localhost", path=str(tmpdir))
+    result = run("bw apply --skip tag:nope -- localhost", path=str(tmpdir))
     assert result[2] == 0
     assert not exists(join(str(tmpdir), "foo"))
 
@@ -149,7 +147,7 @@ def test_skip_type(tmpdir):
             },
         },
     )
-    result = run("bw apply --skip file: localhost", path=str(tmpdir))
+    result = run("bw apply --skip file: -- localhost", path=str(tmpdir))
     assert result[2] == 0
     assert not exists(join(str(tmpdir), "foo"))
 
@@ -179,7 +177,7 @@ def test_skip_trigger(tmpdir):
             },
         },
     )
-    result = run("bw apply --skip tag:nope localhost", path=str(tmpdir))
+    result = run("bw apply --skip tag:nope -- localhost", path=str(tmpdir))
     assert result[2] == 0
     assert not exists(join(str(tmpdir), "foo"))
     assert not exists(join(str(tmpdir), "bar"))
