@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from os import mkdir, readlink, symlink
 from os.path import join
 
