@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
 from re import search
 
 from bundlewrap.utils.testing import host_os, make_repo, run
@@ -28,12 +26,12 @@ def test_add_lock_apply_remove(tmpdir):
             },
         },
     )
-    run("rm -f /tmp/bw_test_lock_add")
-    stdout, stderr, rcode = run("BW_IDENTITY=jdoe bw lock add -c höhöhö -e 1m -i file:/tmp/bw_test_lock_add localhost", path=str(tmpdir))
+    run("sudo rm -f /tmp/bw_test_lock_add")
+    stdout, stderr, rcode = run("BW_IDENTITY=jdoe bw lock add -c höhöhö -e 1m -i file:/tmp/bw_test_lock_add -- localhost", path=str(tmpdir))
     assert rcode == 0
     lock_id = get_lock_id(stdout.decode('utf-8'))
     assert len(lock_id) == 4
-    stdout, stderr, rcode = run("bw apply localhost", path=str(tmpdir))
+    stdout, stderr, rcode = run("bw -d apply localhost", path=str(tmpdir))
     assert rcode == 0
     stdout, stderr, rcode = run("cat /tmp/bw_test_lock_add", path=str(tmpdir))
     assert rcode != 0
