@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from json import loads
 from os.path import join
 
@@ -46,6 +43,7 @@ def test_merge(tmpdir):
         tmpdir,
         nodes={
             "node1": {
+                'groups': {"group1"},
                 'metadata': {
                     "foo": {
                         "bar": "baz",
@@ -55,7 +53,6 @@ def test_merge(tmpdir):
         },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     "ding": 5,
                     "foo": {
@@ -78,65 +75,6 @@ def test_merge(tmpdir):
     assert rcode == 0
 
 
-def test_template_node(tmpdir):
-    make_repo(
-        tmpdir,
-        nodes={
-            "node1": {
-                'template_node': "node2",
-            },
-            "node2": {
-                'metadata': {
-                    "foo": 2,
-                },
-            },
-        },
-        groups={
-            "group1": {
-                'members': ["node1"],
-                'metadata': {
-                    "foo": 3,
-                },
-            },
-        },
-    )
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert loads(stdout.decode()) == {"foo": 2}
-    assert stderr == b""
-    assert rcode == 0
-
-
-def test_template_node_override(tmpdir):
-    make_repo(
-        tmpdir,
-        nodes={
-            "node1": {
-                'metadata': {
-                    "foo": 1,
-                },
-                'template_node': "node2",
-            },
-            "node2": {
-                'metadata': {
-                    "foo": 2,
-                },
-            },
-        },
-        groups={
-            "group1": {
-                'members': ["node1"],
-                'metadata': {
-                    "foo": 3,
-                },
-            },
-        },
-    )
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert loads(stdout.decode()) == {"foo": 1}
-    assert stderr == b""
-    assert rcode == 0
-
-
 def test_metadatapy(tmpdir):
     make_repo(
         tmpdir,
@@ -144,21 +82,32 @@ def test_metadatapy(tmpdir):
         nodes={
             "node1": {
                 'bundles': ["test"],
-                'metadata': {"foo": "bar"},
+                'metadata': {
+                    "foo": {
+                        "bar": "shizzle",
+                    },
+                },
             },
         },
     )
     with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
         f.write(
-"""@metadata_processor
+"""@metadata_reactor
 def foo(metadata):
-    metadata["baz"] = node.name
-    return metadata, DONE
+    return {
+        "baz": node.name,
+        "frob": metadata.get("foo/bar", "shnozzle") + "ay",
+        "gob": metadata.get("shlop", "mop"),
+    }
 """)
     stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
     assert loads(stdout.decode()) == {
         "baz": "node1",
-        "foo": "bar",
+        "foo": {
+            "bar": "shizzle",
+        },
+        "frob": "shizzleay",
+        "gob": "mop",
     }
     assert stderr == b""
     assert rcode == 0
@@ -177,16 +126,14 @@ def test_metadatapy_defaults(tmpdir):
     )
     with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
         f.write(
-"""@metadata_processor
-def foo(metadata):
-    return {
-        "foo": "baz",
-        "baz": "foo",
-    }, DONE, DEFAULTS
+"""defaults = {
+    "baz": node.name,
+    "foo": "baz",
+}
 """)
     stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
     assert loads(stdout.decode()) == {
-        "baz": "foo",
+        "baz": "node1",
         "foo": "bar",
     }
     assert stderr == b""
@@ -212,14 +159,12 @@ nodes = {
 """)
     with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
         f.write(
-"""@metadata_processor
-def foo(metadata):
-    return {
-        "foo": {
-            "bar": "frob",
-            "baz": "gobble",
-        },
-    }, DONE, DEFAULTS
+"""defaults = {
+    "foo": {
+        "bar": "frob",
+        "baz": "gobble",
+    },
+}
 """)
     stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
     assert loads(stdout.decode()) == {
@@ -242,190 +187,22 @@ def test_metadatapy_update(tmpdir):
     )
     with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
         f.write(
-"""@metadata_processor
+"""@metadata_reactor
 def foo(metadata):
     return {
-        "foo": "baz",
         "baz": "foo",
-    }, DONE, OVERWRITE
+        "foo": "baz",
+    }
 """)
     stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
     assert loads(stdout.decode()) == {
         "baz": "foo",
-        "foo": "baz",
+        "foo": "bar",
     }
     assert stderr == b""
     assert rcode == 0
 
 
-def test_metadatapy_invalid_number_of_elements(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return metadata
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
-def test_metadatapy_invalid_first_element_not_dict(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return DONE, metadata
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
-def test_metadatapy_invalid_defaults_plus_original_dict(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return metadata, DONE, DEFAULTS
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
-def test_metadatapy_invalid_overwrite_plus_original_dict(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return metadata, DONE, OVERWRITE
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
-def test_metadatapy_invalid_option(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return metadata, DONE, 1000
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
-def test_metadatapy_invalid_done_and_again(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return metadata, DONE, RUN_ME_AGAIN
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
-def test_metadatapy_invalid_no_done_or_again(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return {}, DEFAULTS
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
-def test_metadatapy_invalid_defaults_and_overwrite(tmpdir):
-    make_repo(
-        tmpdir,
-        bundles={"test": {}},
-        nodes={
-            "node1": {
-                'bundles': ["test"],
-                'metadata': {"foo": "bar"},
-            },
-        },
-    )
-    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
-        f.write(
-"""@metadata_processor
-def foo(metadata):
-    return {}, DEFAULTS, OVERWRITE, DONE
-""")
-    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
-    assert rcode != 0
-
-
 def test_table(tmpdir):
     make_repo(
         tmpdir,
@@ -451,27 +228,311 @@ def test_table(tmpdir):
                 },
             },
         },
-        groups={
-            "all": {
-                'members': ["node1", "node2"],
-            },
-        },
+        groups={"all": {'member_patterns': {r".*"}}},
     )
-    stdout, stderr, rcode = run("BW_TABLE_STYLE=grep bw metadata --table all foo_dict bar, foo_list, foo_int, foo_umlaut", path=str(tmpdir))
-    assert stdout.decode('utf-8') == """node\tfoo_dict bar\tfoo_list\tfoo_int\tfoo_umlaut
-node1\tbaz\tbar, 1\t47\tföö
-node2\t<missing>\t\t-3\tfüü
+    stdout, stderr, rcode = run("BW_TABLE_STYLE=grep bw metadata all -k foo_dict/bar foo_list foo_int foo_umlaut", path=str(tmpdir))
+    assert stdout.decode('utf-8') == """node\tfoo_dict/bar\tfoo_int\tfoo_list\tfoo_umlaut
+node1\tbaz\t47\tbar, 1\tföö
+node2\t<missing>\t-3\t\tfüü
 """
     assert stderr == b""
     assert rcode == 0
 
 
-def test_table_no_key(tmpdir):
+def test_metadatapy_merge_order(tmpdir):
     make_repo(
         tmpdir,
+        bundles={"test": {}},
         nodes={
-            "node1": {},
+            "node1": {
+                'bundles': ["test"],
+                'groups': {"group1"},
+                'metadata': {
+                    "four": "node",
+                },
+            },
+        },
+        groups={
+            "group1": {
+                'metadata': {
+                    "three": "group",
+                    "four": "group",
+                },
+            },
         },
     )
-    stdout, stderr, rcode = run("bw metadata --table node1", path=str(tmpdir))
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""defaults = {
+    "one": "defaults",
+    "two": "defaults",
+    "three": "defaults",
+    "four": "defaults",
+}
+
+@metadata_reactor
+def foo_reactor(metadata):
+    return {
+        "two": "reactor",
+        "three": "reactor",
+        "four": "reactor",
+    }
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "one": "defaults",
+        "two": "reactor",
+        "three": "group",
+        "four": "node",
+    }
+    assert stderr == b""
+    assert rcode == 0
+
+
+def test_metadatapy_static_reorder(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+                'metadata': {
+                    "foo": "bar",
+                    "frob": "flup",
+                },
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor
+def foo_reactor(metadata):
+    return {
+        "foo": "overwritten",
+        "baz": metadata.get("frob"),
+    }
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "foo": "bar",
+        "frob": "flup",
+        "baz": "flup",
+    }
+    assert stderr == b""
+    assert rcode == 0
+
+
+def test_metadatapy_reactor_keyerror_from_metastack(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""
+@metadata_reactor
+def foo_reactor(metadata):
+    return {'foo': metadata.get('bar')}
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
     assert rcode == 1
+    assert b"node1" in stderr
+    assert b"foo_reactor" in stderr
+    assert b"'bar'" in stderr
+
+
+def test_metadatapy_reactor_keyerror_from_dict(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""
+@metadata_reactor
+def foo_reactor(metadata):
+    x = {}['baz']
+    return {'x': x}
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert rcode == 1
+    assert b"node1" in stderr
+    assert b"foo_reactor" in stderr
+    assert b"'baz'" in stderr
+
+
+def test_metadatapy_reactor_keyerror_fixed(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""
+@metadata_reactor
+def one(metadata):
+    return {'one': True}
+
+@metadata_reactor
+def two(metadata):
+    return {'two': metadata.get('one')}
+
+@metadata_reactor
+def three(metadata):
+    return {'three': metadata.get('two')}
+
+@metadata_reactor
+def four(metadata):
+    return {'four': metadata.get('three')}
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "one": True,
+        "two": True,
+        "three": True,
+        "four": True,
+    }
+    assert stderr == b""
+    assert rcode == 0
+
+
+def test_metadatapy_infinite_loop(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""
+@metadata_reactor
+def plusone(metadata):
+    return {'foo': metadata.get('bar', 0) + 1 }
+
+@metadata_reactor
+def plustwo(metadata):
+    return {'bar': metadata.get('foo', 0) + 2 }
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert rcode == 1
+
+
+def test_metadatapy_no_self_react(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""
+@metadata_reactor
+def reactor1(metadata):
+    assert not metadata.get('broken', False)
+    return {'broken': True}
+
+@metadata_reactor
+def reactor2(metadata):
+    # just to make sure reactor1 runs again
+    return {'again': True}
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "broken": True,
+        "again": True,
+    }
+
+
+def test_own_node_metadata(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+                'metadata': {'number': 47},
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""
+@metadata_reactor
+def reactor1(metadata):
+    return {'plusone': node.metadata.get('number') + 1}
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "number": 47,
+        "plusone": 48,
+    }
+
+
+def test_other_node_metadata(tmpdir):
+    make_repo(
+        tmpdir,
+        bundles={"test": {}},
+        nodes={
+            "node1": {
+                'bundles': ["test"],
+                'metadata': {'number': 47},
+            },
+            "node2": {
+                'bundles': ["test"],
+                'metadata': {'number': 42},
+            },
+            "node3": {
+                'bundles': ["test"],
+                'metadata': {'number': 23},
+            },
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "test", "metadata.py"), 'w') as f:
+        f.write(
+"""
+@metadata_reactor
+def reactor1(metadata):
+    numbers = set()
+    for n in repo.nodes:
+        if n != node:
+            numbers.add(n.metadata.get('number'))
+    return {'other_numbers': numbers}
+""")
+    stdout, stderr, rcode = run("bw metadata node1", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "number": 47,
+        "other_numbers": [23, 42],
+    }
+    stdout, stderr, rcode = run("bw metadata node2", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "number": 42,
+        "other_numbers": [23, 47],
+    }
+    stdout, stderr, rcode = run("bw metadata node3", path=str(tmpdir))
+    assert loads(stdout.decode()) == {
+        "number": 23,
+        "other_numbers": [42, 47],
+    }
