@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from bundlewrap.utils.testing import make_repo, run
 
 
@@ -27,15 +24,16 @@ def test_nondeterministic(tmpdir):
     )
 
     stdout, stderr, rcode = run("bw stats", path=str(tmpdir))
-    assert stdout == """╭───────┬─────────────────────╮
-│ count │ type                │
-├───────┼─────────────────────┤
-│     1 │ nodes               │
-│     0 │ groups              │
-│     1 │ bundles             │
-│     0 │ metadata processors │
-│     2 │ items               │
-├───────┼─────────────────────┤
-│     2 │ file                │
-╰───────┴─────────────────────╯
+    assert stdout == """╭───────┬───────────────────╮
+│ count │ type              │
+├───────┼───────────────────┤
+│     1 │ nodes             │
+│     0 │ groups            │
+│     1 │ bundles           │
+│     0 │ metadata defaults │
+│     0 │ metadata reactors │
+│     2 │ items             │
+├───────┼───────────────────┤
+│     2 │ file              │
+╰───────┴───────────────────╯
 """.encode('utf-8')
