@@ -115,6 +115,30 @@ def test_circular_dep_self(tmpdir):
     assert run("bw test -I", path=str(tmpdir))[2] == 1
 
 
+def test_unknown_tag(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': ["bundle1"],
+            },
+        },
+        bundles={
+            "bundle1": {
+                "files": {
+                    "/foo": {
+                        'content': "none",
+                        'needs': {
+                            "tag:bar",
+                        },
+                    },
+                },
+            },
+        },
+    )
+    assert run("bw test -I", path=str(tmpdir))[2] == 0
+
+
 def test_circular_trigger_self(tmpdir):
     make_repo(
         tmpdir,
@@ -200,10 +224,16 @@ def test_group_loop(tmpdir):
 def test_group_metadata_collision(tmpdir):
     make_repo(
         tmpdir,
-        nodes={"node1": {}},
+        nodes={
+            "node1": {
+                'groups': {
+                    "group1",
+                    "group3",
+                },
+            },
+        },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': {
                         'baz': 1,
@@ -220,9 +250,7 @@ def test_group_metadata_collision(tmpdir):
                 },
                 'subgroups': ["group3"],
             },
-            "group3": {
-                'members': ["node1"],
-            },
+            "group3": {},
         },
     )
     assert run("bw test -M", path=str(tmpdir))[2] == 1
@@ -231,10 +259,16 @@ def test_group_metadata_collision(tmpdir):
 def test_group_metadata_collision_subgroups(tmpdir):
     make_repo(
         tmpdir,
-        nodes={"node1": {}},
+        nodes={
+            "node1": {
+                'groups': {
+                    "group1",
+                    "group3",
+                },
+            },
+        },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': {
                         'baz': 1,
@@ -251,9 +285,7 @@ def test_group_metadata_collision_subgroups(tmpdir):
                 },
                 'subgroups': ["group1", "group3"],
             },
-            "group3": {
-                'members': ["node1"],
-            },
+            "group3": {},
         },
     )
     assert run("bw test -M", path=str(tmpdir))[2] == 0
@@ -262,16 +294,21 @@ def test_group_metadata_collision_subgroups(tmpdir):
 def test_group_metadata_collision_list(tmpdir):
     make_repo(
         tmpdir,
-        nodes={"node1": {}},
+        nodes={
+            "node1": {
+                'groups': {
+                    "group1",
+                    "group2",
+                },
+            },
+        },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': [1],
                 },
             },
             "group2": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': [2],
                 },
@@ -284,16 +321,21 @@ def test_group_metadata_collision_list(tmpdir):
 def test_group_metadata_collision_dict(tmpdir):
     make_repo(
         tmpdir,
-        nodes={"node1": {}},
+        nodes={
+            "node1": {
+                'groups': {
+                    "group1",
+                    "group2",
+                },
+            },
+        },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': {'bar': 1},
                 },
             },
             "group2": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': 2,
                 },
@@ -306,16 +348,21 @@ def test_group_metadata_collision_dict(tmpdir):
 def test_group_metadata_collision_dict_ok(tmpdir):
     make_repo(
         tmpdir,
-        nodes={"node1": {}},
+        nodes={
+            "node1": {
+                'groups': {
+                    "group1",
+                    "group2",
+                },
+            },
+        },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': {'bar': 1},
                 },
             },
             "group2": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': {'baz': 2},
                 },
@@ -328,16 +375,21 @@ def test_group_metadata_collision_dict_ok(tmpdir):
 def test_group_metadata_collision_set(tmpdir):
     make_repo(
         tmpdir,
-        nodes={"node1": {}},
+        nodes={
+            "node1": {
+                'groups': {
+                    "group1",
+                    "group2",
+                },
+            },
+        },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': set([1]),
                 },
             },
             "group2": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': 2,
                 },
@@ -350,16 +402,21 @@ def test_group_metadata_collision_set(tmpdir):
 def test_group_metadata_collision_set_ok(tmpdir):
     make_repo(
         tmpdir,
-        nodes={"node1": {}},
+        nodes={
+            "node1": {
+                'groups': {
+                    "group1",
+                    "group2",
+                },
+            },
+        },
         groups={
             "group1": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': set([1]),
                 },
             },
             "group2": {
-                'members': ["node1"],
                 'metadata': {
                     'foo': set([2]),
                 },
@@ -369,6 +426,184 @@ def test_group_metadata_collision_set_ok(tmpdir):
     assert run("bw test -M", path=str(tmpdir))[2] == 0
 
 
+def test_defaults_metadata_collision(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1", "bundle2"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+            "bundle2": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""defaults = {
+    "foo": "bar",
+}
+""")
+    with open(join(str(tmpdir), "bundles", "bundle2", "metadata.py"), 'w') as f:
+        f.write(
+"""defaults = {
+    "foo": "baz",
+}
+""")
+    stdout, stderr, rcode = run("bw test -M", path=str(tmpdir))
+    assert rcode == 1
+    assert b"foo" in stderr
+
+
+def test_defaults_metadata_collision_nested(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1", "bundle2"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+            "bundle2": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""defaults = {
+    "foo": {"bar": "baz"},
+}
+""")
+    with open(join(str(tmpdir), "bundles", "bundle2", "metadata.py"), 'w') as f:
+        f.write(
+"""defaults = {
+    "foo": {"bar": "frob"},
+}
+""")
+    stdout, stderr, rcode = run("bw test -M", path=str(tmpdir))
+    assert rcode == 1
+    assert b"foo/bar" in stderr
+
+
+def test_defaults_metadata_collision_ok(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1", "bundle2"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+            "bundle2": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""defaults = {
+    "foo": {"bar"},
+}
+""")
+    with open(join(str(tmpdir), "bundles", "bundle2", "metadata.py"), 'w') as f:
+        f.write(
+"""defaults = {
+    "foo": {"baz"},
+}
+""")
+    assert run("bw test -M", path=str(tmpdir))[2] == 0
+
+
+def test_reactor_metadata_collision(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1", "bundle2"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+            "bundle2": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor
+def foo(metadata):
+    return {"foo": 1}
+""")
+    with open(join(str(tmpdir), "bundles", "bundle2", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor
+def foo(metadata):
+    return {"foo": 2}
+""")
+    stdout, stderr, rcode = run("bw test -M", path=str(tmpdir))
+    assert rcode == 1
+    assert b"foo" in stderr
+
+
+def test_reactor_metadata_collision_nested(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1", "bundle2"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+            "bundle2": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor
+def foo(metadata):
+    return {"foo": {"bar": "1"}}
+""")
+    with open(join(str(tmpdir), "bundles", "bundle2", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor
+def foo(metadata):
+    return {"foo": {"bar": "2"}}
+""")
+    stdout, stderr, rcode = run("bw test -M", path=str(tmpdir))
+    assert rcode == 1
+    assert b"foo/bar" in stderr
+
+
+def test_reactor_metadata_collision_nested_mixed(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1", "bundle2"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+            "bundle2": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor
+def foo(metadata):
+    return {"foo": {"bar": {True}}}
+""")
+    with open(join(str(tmpdir), "bundles", "bundle2", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor
+def foo(metadata):
+    return {"foo": {"bar": [False]}}
+""")
+    stdout, stderr, rcode = run("bw test -M", path=str(tmpdir))
+    assert rcode == 1
+    assert b"foo/bar" in stderr
+
+
 def test_fault_missing(tmpdir):
     make_repo(
         tmpdir,
@@ -392,6 +627,30 @@ def test_fault_missing(tmpdir):
     assert run("bw test -iI", path=str(tmpdir))[2] == 0
 
 
+def test_fault_missing_content(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': ["bundle1"],
+            },
+        },
+        bundles={
+            "bundle1": {}
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "items.py"), 'w') as f:
+        f.write("""
+files = {
+    "/foo": {
+        'content': repo.vault.decrypt("bzzt", key="unavailable"),
+    },
+}
+""")
+    assert run("bw test -I", path=str(tmpdir))[2] == 1
+    assert run("bw test -iI", path=str(tmpdir))[2] == 0
+
+
 def test_metadata_determinism_ok(tmpdir):
     make_repo(
         tmpdir,
@@ -405,10 +664,9 @@ def test_metadata_determinism_ok(tmpdir):
         },
     )
     with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
-        f.write("""@metadata_processor
+        f.write("""@metadata_reactor
 def test(metadata):
-    metadata['test'] = 1
-    return metadata, DONE
+    return {'test': 1}
 """)
     assert run("bw test -m 3", path=str(tmpdir))[2] == 0
 
@@ -427,11 +685,11 @@ def test_metadata_determinism_broken(tmpdir):
     )
     with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
         f.write("""from random import randint
+n = randint(1, 99999)
 
-@metadata_processor
+@metadata_reactor
 def test(metadata):
-    metadata.setdefault('test', randint(1, 99999))
-    return metadata, DONE
+    return {'test': n}
 """)
     assert run("bw test -m 3", path=str(tmpdir))[2] == 1
 
@@ -484,11 +742,11 @@ def test_unknown_subgroup(tmpdir):
     make_repo(
         tmpdir,
         nodes={
-            "node1": {},
+            "node1": {'groups': {"group2"}},
         },
         groups={
             "group1": {'subgroups': ["missing-group"]},
-            "group2": {'members': ["node1"]},
+            "group2": {},
         },
     )
     assert run("bw test", path=str(tmpdir))[2] == 1
@@ -500,11 +758,11 @@ def test_empty_group(tmpdir):
     make_repo(
         tmpdir,
         nodes={
-            "node1": {},
+            "node1": {'groups': {"group2"}},
         },
         groups={
             "group1": {},
-            "group2": {'members': ["node1"]},
+            "group2": {},
         },
     )
     assert run("bw test", path=str(tmpdir))[2] == 0
@@ -586,57 +844,6 @@ def test_group_user_dep_deleted_gid(tmpdir):
     assert run("bw test -I", path=str(tmpdir))[2] == 1
 
 
-def test_secret_identifier_only_once(tmpdir):
-    make_repo(
-        tmpdir,
-        nodes={
-            "node1": {
-                'bundles': ["bundle1"],
-            },
-        },
-        bundles={
-            "bundle1": {
-                'files': {
-                    "/test": {
-                        'content': "${repo.vault.password_for('testing')}",
-                        'content_type': 'mako',
-                    },
-                },
-            },
-        },
-    )
-    assert run("bw test -s ''", path=str(tmpdir))[2] == 1
-    assert run("bw test -s 'test'", path=str(tmpdir))[2] == 0
-    assert run("bw test -s 'test,foo'", path=str(tmpdir))[2] == 0
-
-
-def test_secret_identifier_twice(tmpdir):
-    make_repo(
-        tmpdir,
-        nodes={
-            "node1": {
-                'bundles': ["bundle1"],
-            },
-            "node2": {
-                'bundles': ["bundle1"],
-            },
-        },
-        bundles={
-            "bundle1": {
-                'files': {
-                    "/test": {
-                        'content': "${repo.vault.password_for('testing')}",
-                        'content_type': 'mako',
-                    },
-                },
-            },
-        },
-    )
-    assert run("bw test -s ''", path=str(tmpdir))[2] == 0
-    assert run("bw test -s 'test'", path=str(tmpdir))[2] == 0
-    assert run("bw test -s 'test,foo'", path=str(tmpdir))[2] == 0
-
-
 def test_reverse_dummy_dep(tmpdir):
     make_repo(
         tmpdir,
