@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from os.path import join
 
 from bundlewrap.utils.testing import host_os, make_repo, run
